import * as i0 from '@angular/core';
import { EventEmitter, Directive, Inject, HostBinding, ContentChildren, forwardRef, Input, Output, HostListener, NgModule } from '@angular/core';
import { DOCUMENT, CommonModule } from '@angular/common';
import { combineLatest, map, takeUntil, filter, mergeMap, BehaviorSubject, Subject, take } from 'rxjs';
import * as i1 from '@angular/animations';
import { style, animate } from '@angular/animations';

class BsSwipeContainerDirective {
    animationBuilder;
    constructor(element, animationBuilder, document) {
        this.animationBuilder = animationBuilder;
        this.containerElement = element;
        this.document = document;
        this.offset$ = combineLatest([this.startTouch$, this.lastTouch$, this.imageIndex$, this.isViewInited$])
            .pipe(map(([startTouch, lastTouch, imageIndex, isViewInited]) => {
            if (!isViewInited) {
                return (-imageIndex * 100);
            }
            else if (!!startTouch && !!lastTouch) {
                return (-imageIndex * 100 + (lastTouch.position.x - startTouch.position.x) / this.containerElement.nativeElement.clientWidth * 100);
            }
            else {
                return (-imageIndex * 100);
            }
        }));
        this.padLeft$ = this.swipes$.pipe(map(swipes => {
            if (!swipes) {
                return 0;
            }
            let count = 0;
            for (const s of swipes) {
                if (!s.offside) {
                    break;
                }
                else {
                    count++;
                }
            }
            return count;
        }));
        this.padRight$ = this.swipes$.pipe(map(swipes => {
            if (!swipes) {
                return 0;
            }
            let count = 0;
            for (const s of swipes.toArray().reverse()) {
                if (!s.offside) {
                    break;
                }
                else {
                    count++;
                }
            }
            return count;
        }));
        this.offsetLeft$ = combineLatest([this.offset$, this.padLeft$])
            .pipe(map(([offset, padLeft]) => {
            return offset - padLeft * 100;
        }));
        this.offsetRight$ = combineLatest([this.offset$, this.padLeft$, this.padRight$])
            .pipe(map(([offset, padLeft, padRight]) => {
            return -(offset - padLeft * 100) - (padRight - 1) * 100;
        }));
        this.offsetLeft$
            .pipe(takeUntil(this.destroyed$))
            .subscribe((offsetLeft) => this.offsetLeft = offsetLeft);
        this.offsetRight$
            .pipe(takeUntil(this.destroyed$))
            .subscribe((offsetRight) => this.offsetRight = offsetRight);
        this.imageIndex$
            .pipe(takeUntil(this.destroyed$))
            .subscribe((imageIndex) => this.imageIndexChange.emit(imageIndex));
        this.actualSwipes$ = this.swipes$
            .pipe(map(swipes => {
            if (swipes) {
                return swipes.filter(swipe => !swipe.offside);
            }
            else {
                return [];
            }
        }));
        this.slideHeights$ = this.actualSwipes$
            .pipe(filter(swipes => !!swipes))
            // .pipe(map(swipes => <QueryList<BsSwipeDirective>>swipes))
            .pipe(mergeMap(swipes => combineLatest(swipes.map(swipe => swipe.slideHeight$))));
        this.currentSlideHeight$ = combineLatest([this.slideHeights$, this.imageIndex$])
            .pipe(map(([slideHeights, imageIndex]) => {
            const maxHeight = Math.max(...slideHeights);
            const currHeight = slideHeights[imageIndex] ?? maxHeight;
            return maxHeight - (maxHeight - currHeight) /* / 2*/;
        }));
    }
    offsetLeft = null;
    offsetRight = null;
    offsetTop = null;
    offsetBottom = null;
    set swipes(value) {
        setTimeout(() => this.swipes$.next(value));
    }
    minimumOffset = 50;
    //#region ImageIndex
    get imageIndex() {
        return this.imageIndex$.value;
    }
    set imageIndex(value) {
        this.imageIndex$.next(value);
    }
    imageIndexChange = new EventEmitter();
    //#endregion
    actualSwipes$;
    isViewInited$ = new BehaviorSubject(false);
    destroyed$ = new Subject();
    startTouch$ = new BehaviorSubject(null);
    lastTouch$ = new BehaviorSubject(null);
    swipes$ = new BehaviorSubject(null);
    imageIndex$ = new BehaviorSubject(0);
    slideHeights$;
    currentSlideHeight$;
    pendingAnimation;
    containerElement;
    document;
    offset$;
    offsetLeft$;
    offsetRight$;
    padLeft$;
    padRight$;
    ngAfterViewInit() {
        this.isViewInited$.next(true);
    }
    ngOnDestroy() {
        this.destroyed$.next(true);
    }
    animateToIndexByDx(dx) {
        combineLatest([this.imageIndex$, this.actualSwipes$]).pipe(take(1))
            .subscribe(([imageIndex, actualSwipes]) => {
            const direction = dx > 0 ? 'left' : 'right';
            let newIndex;
            if (Math.abs(dx) < this.minimumOffset) {
                newIndex = imageIndex;
            }
            else {
                newIndex = imageIndex + (direction === 'right' ? 1 : -1);
            }
            this.animateToIndex(imageIndex, newIndex, dx, actualSwipes?.length ?? 1);
        });
    }
    animateToIndex(oldIndex, newIndex, dx, totalSlides) {
        this.pendingAnimation = this.animationBuilder.build([
            style({ 'margin-left': (-(oldIndex + 1) * this.containerElement.nativeElement.clientWidth + dx) + 'px', 'margin-right': ((oldIndex + 1) * this.containerElement.nativeElement.clientWidth - dx) + 'px' }),
            animate('500ms ease', style({ 'margin-left': (-(newIndex + 1) * this.containerElement.nativeElement.clientWidth) + 'px', 'margin-right': ((newIndex + 1) * this.containerElement.nativeElement.clientWidth) + 'px' })),
        ]).create(this.containerElement.nativeElement);
        this.pendingAnimation.onDone(() => {
            // Correct the image index
            if (newIndex === -1) {
                this.imageIndex$.next(totalSlides - 1);
            }
            else if (newIndex === totalSlides) {
                this.imageIndex$.next(0);
            }
            else {
                this.imageIndex$.next(newIndex);
            }
            this.startTouch$.next(null);
            this.lastTouch$.next(null);
            this.pendingAnimation?.destroy();
            this.pendingAnimation = undefined;
        });
        this.pendingAnimation.play();
    }
    onSwipe(dx) {
        this.animateToIndexByDx(dx);
    }
    previous() {
        this.pendingAnimation?.finish();
        combineLatest([this.actualSwipes$, this.imageIndex$]).pipe(take(1)).subscribe(([actualSwipes, imageIndex]) => {
            setTimeout(() => this.animateToIndex(imageIndex, imageIndex - 1, 0, actualSwipes?.length ?? 1), 20);
        });
    }
    next() {
        this.pendingAnimation?.finish();
        combineLatest([this.actualSwipes$, this.imageIndex$]).pipe(take(1)).subscribe(([actualSwipes, imageIndex]) => {
            setTimeout(() => this.animateToIndex(imageIndex, imageIndex + 1, 0, actualSwipes?.length ?? 1), 20);
        });
    }
    goto(index) {
        combineLatest([this.actualSwipes$, this.imageIndex$]).pipe(take(1)).subscribe(([actualSwipes, imageIndex]) => {
            this.pendingAnimation?.finish();
            setTimeout(() => this.animateToIndex(imageIndex, index, 0, actualSwipes?.length ?? 1), 20);
        });
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: BsSwipeContainerDirective, deps: [{ token: i0.ElementRef }, { token: i1.AnimationBuilder }, { token: DOCUMENT }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.0", type: BsSwipeContainerDirective, selector: "[bsSwipeContainer]", inputs: { minimumOffset: "minimumOffset", imageIndex: "imageIndex" }, outputs: { imageIndexChange: "imageIndexChange" }, host: { properties: { "style.margin-left.%": "this.offsetLeft", "style.margin-right.%": "this.offsetRight", "style.margin-top.%": "this.offsetTop", "style.margin-bottom.%": "this.offsetBottom" } }, queries: [{ propertyName: "swipes", predicate: i0.forwardRef(function () { return BsSwipeDirective; }) }], exportAs: ["bsSwipeContainer"], ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: BsSwipeContainerDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bsSwipeContainer]',
                    exportAs: 'bsSwipeContainer'
                }]
        }], ctorParameters: function () { return [{ type: i0.ElementRef }, { type: i1.AnimationBuilder }, { type: undefined, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }]; }, propDecorators: { offsetLeft: [{
                type: HostBinding,
                args: ['style.margin-left.%']
            }], offsetRight: [{
                type: HostBinding,
                args: ['style.margin-right.%']
            }], offsetTop: [{
                type: HostBinding,
                args: ['style.margin-top.%']
            }], offsetBottom: [{
                type: HostBinding,
                args: ['style.margin-bottom.%']
            }], swipes: [{
                type: ContentChildren,
                args: [forwardRef(() => BsSwipeDirective)]
            }], minimumOffset: [{
                type: Input
            }], imageIndex: [{
                type: Input
            }], imageIndexChange: [{
                type: Output
            }] } });

class BsSwipeDirective {
    container;
    constructor(container, element) {
        this.container = container;
        this.element = element;
    }
    element;
    observer;
    slideHeight$ = new BehaviorSubject(0);
    //#region Offside
    offside = false;
    //#endregion
    alignTopClass = true;
    onTouchStart(ev) {
        if (ev.touches.length === 1) {
            ev.preventDefault();
            this.container.pendingAnimation?.finish();
            setTimeout(() => {
                this.container.startTouch$.next({
                    position: {
                        x: ev.touches[0].clientX,
                        y: ev.touches[0].clientY,
                    },
                    timestamp: Date.now(),
                });
                this.container.lastTouch$.next({
                    position: {
                        x: ev.touches[0].clientX,
                        y: ev.touches[0].clientY,
                    },
                    isTouching: true,
                });
            }, 20);
        }
    }
    onTouchMove(ev) {
        this.container.lastTouch$.next({
            position: {
                x: ev.touches[0].clientX,
                y: ev.touches[0].clientY,
            },
            isTouching: true,
        });
    }
    onTouchEnd(ev) {
        combineLatest([this.container.startTouch$, this.container.lastTouch$])
            .pipe(filter(([startTouch, lastTouch]) => !!startTouch && !!lastTouch))
            .pipe(take(1))
            .subscribe(([startTouch, lastTouch]) => {
            if (!!startTouch && !!lastTouch) {
                const dx = lastTouch.position.x - startTouch.position.x;
                this.container.onSwipe(dx);
            }
        });
    }
    ngAfterViewInit() {
        this.observer = new ResizeObserver((entries) => {
            this.slideHeight$.next(entries[0].contentRect.height);
        });
        this.observer.observe(this.element.nativeElement);
    }
    ngOnDestroy() {
        if (this.observer) {
            this.observer.unobserve(this.element.nativeElement);
            this.observer.disconnect();
        }
    }
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: BsSwipeDirective, deps: [{ token: BsSwipeContainerDirective }, { token: i0.ElementRef }], target: i0.ɵɵFactoryTarget.Directive });
    static ɵdir = i0.ɵɵngDeclareDirective({ minVersion: "14.0.0", version: "16.0.0", type: BsSwipeDirective, selector: "[bsSwipe]", inputs: { offside: "offside" }, host: { listeners: { "touchstart": "onTouchStart($event)", "touchmove": "onTouchMove($event)", "touchend": "onTouchEnd($event)" }, properties: { "class.align-top": "this.alignTopClass" } }, ngImport: i0 });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: BsSwipeDirective, decorators: [{
            type: Directive,
            args: [{
                    selector: '[bsSwipe]'
                }]
        }], ctorParameters: function () { return [{ type: BsSwipeContainerDirective }, { type: i0.ElementRef }]; }, propDecorators: { offside: [{
                type: Input
            }], alignTopClass: [{
                type: HostBinding,
                args: ['class.align-top']
            }], onTouchStart: [{
                type: HostListener,
                args: ['touchstart', ['$event']]
            }], onTouchMove: [{
                type: HostListener,
                args: ['touchmove', ['$event']]
            }], onTouchEnd: [{
                type: HostListener,
                args: ['touchend', ['$event']]
            }] } });

class BsSwiperModule {
    static ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: BsSwiperModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule });
    static ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "16.0.0", ngImport: i0, type: BsSwiperModule, declarations: [BsSwipeDirective, BsSwipeContainerDirective], imports: [CommonModule], exports: [BsSwipeDirective, BsSwipeContainerDirective] });
    static ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: BsSwiperModule, imports: [CommonModule] });
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "16.0.0", ngImport: i0, type: BsSwiperModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule],
                    declarations: [BsSwipeDirective, BsSwipeContainerDirective],
                    exports: [BsSwipeDirective, BsSwipeContainerDirective],
                }]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { BsSwipeContainerDirective, BsSwipeDirective, BsSwiperModule };
//# sourceMappingURL=mintplayer-ng-swiper.mjs.map
